import { NodeManager, NodeManagerOptions } from '../helpers/healthCheck';
import { type CustomLogger, type LogLevel, type LogLevelName } from '../helpers/logger';
import { AdamantApiResult } from '../helpers/validator';
import { MessageType } from '../helpers/constants';
import type { GetAccountBalanceResponseDto, GetAccountInfoResponseDto, GetAccountVotesResponseDto, GetBlockInfoResponseDto, GetBlocksResponseDto, GetBroadhashResponseDto, GetChatMessagesResponseDto, GetChatRoomsResponseDto, GetDelegateResponseDto, GetDelegateStatsResponseDto, GetDelegatesCountResponseDto, GetDelegatesResponseDto, GetEpochResponseDto, GetHeightResponseDto, GetLoadingStatusResponseDto, GetMilestoneResponseDto, GetNethashResponseDto, GetNetworkInfoResponseDto, GetNextForgersResponseDto, GetNodeStatusResponseDto, GetNodeVersionResponseDto, GetPeersResponseDto, GetPingStatusResponseDto, GetQueuedTransactionsResponseDto, GetRewardResponseDto, GetSyncStatusResponseDto, GetTokenTransferFeeResponseDto, GetTokensTotalSupplyResponseDto, GetTransactionByIdResponseDto, GetTransactionTypesFeesResponseDto, GetTransactionsCountResponseDto, GetTransactionsResponseDto, GetUnconfirmedTransactionByIdResponseDto, GetUnconfirmedTransactionsResponseDto, GetVotersResponseDto, SearchDelegateResponseDto } from './generated';
export type AdamantAddress = `U${string}`;
export interface UsernameObject {
    username: string;
}
export interface PublicKeyObject {
    publicKey: string;
}
export interface AddressObject {
    address: string;
}
/**
 * Object that contains either `address` or `publicKey` field
 */
export type AddressOrPublicKeyObject = AddressObject | PublicKeyObject;
/**
 * Object that contains either `username` or `publicKey` field
 */
export type UsernameOrPublicKeyObject = UsernameObject | PublicKeyObject;
export type TransactionQuery<T extends object> = Partial<T> & {
    or?: Partial<T>;
    and?: Partial<T>;
};
export interface GetBlocksOptions {
    limit?: number;
    offset?: number;
    generatorPublicKey?: string;
    height?: number;
}
export interface GetDelegatesOptions {
    limit?: number;
    offset?: number;
}
export interface TransactionQueryParameters {
    blockId?: number;
    fromHeight?: number;
    toHeight?: number;
    minAmount?: number;
    maxAmount?: number;
    senderId?: string;
    recipientId?: string;
    inId?: string;
    limit?: number;
    offset?: number;
    orderBy?: string;
}
export interface ChatroomsOptions extends TransactionQueryParameters {
    type?: number;
    withoutDirectTransfers?: boolean;
}
export interface TransactionsOptions extends TransactionQueryParameters {
    senderIds?: string[];
    recipientIds?: string[];
    senderPublicKey?: string;
    senderPublicKeys?: string[];
    recipientPublicKey?: string;
    recipientPublicKeys?: string[];
    type?: number;
    types?: number[];
    returnAsset?: 1 | 0;
}
export interface AdamantApiOptions extends NodeManagerOptions {
    nodes: string[];
    maxRetries?: number;
    timeout?: number;
    logger?: CustomLogger;
    logLevel?: LogLevel | LogLevelName;
}
export declare class AdamantApi extends NodeManager {
    maxRetries: number;
    constructor(options: AdamantApiOptions);
    private request;
    /**
     * Makes GET request to ADAMANT network.
     *
     * @details `endpoint` should be in `'accounts/getPublicKey'` format, excluding `'/api/'`
     */
    get<T>(endpoint: string, params?: unknown): Promise<AdamantApiResult<T>>;
    /**
     * Makes POST request to ADAMANT network.
     *
     * @details `endpoint` should be in `'accounts/getPublicKey'` format, excluding `'/api/'`
     */
    post<T>(endpoint: string, options: unknown): Promise<AdamantApiResult<T>>;
    /**
     * Get account Public Key
     */
    getPublicKey(address: AdamantAddress): Promise<string>;
    /**
     * Register new delegate within given username
     *
     * @param username The new delegate's username
     */
    newDelegate(passphrase: string, username: string): Promise<{
        success: boolean;
        error: string;
    } | AdamantApiResult<unknown>>;
    /**
     * Encrypt a message, creates Message transaction, signs it, and broadcasts to ADAMANT network.
     */
    sendMessage(passphrase: string, addressOrPublicKey: string, message: string, type?: MessageType, amount?: number, isAmountInADM?: boolean): Promise<{
        success: boolean;
        error: string;
    } | (Omit<unknown, "success"> & {
        success: true;
    }) | {
        success: boolean;
        errorMessage: string;
    }>;
    /**
     * Send tokens to an account
     */
    sendTokens(passphrase: string, addressOrPublicKey: string, amount: number, isAmountInADM?: boolean): {
        success: boolean;
        error: string;
    } | Promise<AdamantApiResult<unknown>>;
    /**
     * Vote for specific delegates
     *
     * @param votes Array with public keys. For upvote, add leading `+` to delegate's public key. For downvote, add leading `-` to delegate's public key.
     *
     * @example
     * ```
     * voteForDelegate([
     *   '+b3d0c0b99f64d0960324089eb678e90d8bcbb3dd8c73ee748e026f8b9a5b5468',
     *   '-9ef1f6212ae871716cfa2d04e3dc5339e8fe75f89818be21ee1d75004983e2a8'
     * ])
     * ```
     */
    voteForDelegate(passphrase: string, votes: string[]): Promise<{
        success: boolean;
        error: string;
    } | AdamantApiResult<unknown>>;
    /**
     * Get account information by ADAMANT address or Public Key
     */
    getAccountInfo(options: AddressOrPublicKeyObject): Promise<AdamantApiResult<GetAccountInfoResponseDto>>;
    /**
     * Get account balance
     */
    getAccountBalance(address: string): Promise<AdamantApiResult<GetAccountBalanceResponseDto>>;
    /**
     * Get block information by ID
     */
    getBlock(id: string): Promise<AdamantApiResult<GetBlockInfoResponseDto>>;
    /**
     * Get list of blocks
     */
    getBlocks(options?: GetBlocksOptions): Promise<AdamantApiResult<GetBlocksResponseDto>>;
    /**
     * Get list of Chats
     */
    getChats(address: string, options?: TransactionQuery<ChatroomsOptions>): Promise<AdamantApiResult<GetChatRoomsResponseDto>>;
    /**
     * Get messages between two accounts
     */
    getChatMessages(address1: string, address2: string, query?: TransactionQuery<ChatroomsOptions>): Promise<AdamantApiResult<GetChatMessagesResponseDto>>;
    /**
     * Retrieves list of registered ADAMANT delegates
     */
    getDelegates(options: GetDelegatesOptions): Promise<AdamantApiResult<GetDelegatesResponseDto>>;
    /**
     * Get delegate info by `username` or `publicKey`
     */
    getDelegate(options: UsernameOrPublicKeyObject): Promise<AdamantApiResult<GetDelegateResponseDto>>;
    /**
     * Search delegates by `username`
     *
     * @param q - username
     */
    searchDelegates(q: string): Promise<AdamantApiResult<SearchDelegateResponseDto>>;
    /**
     * Get total count of delegates
     */
    getDelegatesCount(): Promise<AdamantApiResult<GetDelegatesCountResponseDto>>;
    /**
     * Get forging activity of a delegate
     */
    getDelegateStats(generatorPublicKey: string): Promise<AdamantApiResult<GetDelegateStatsResponseDto>>;
    /**
     * Returns list of next forgers
     *
     * @param limit count to retrieve
     */
    getNextForgers(limit?: number): Promise<AdamantApiResult<GetNextForgersResponseDto>>;
    /**
     * Gets list of delegate's voters
     *
     * @param publicKey representing delegate's publicKey
     */
    getVoters(publicKey: string): Promise<AdamantApiResult<GetVotersResponseDto>>;
    /**
     * Gets current votes of specific ADAMANT account
     *
     * @param address address of the account to get votes
     */
    getVoteData(address: string): Promise<AdamantApiResult<GetAccountVotesResponseDto>>;
    /**
     * Gets list of connected peer nodes
     */
    getPeers(): Promise<AdamantApiResult<GetPeersResponseDto>>;
    /**
     * Gets loading status
     */
    getLoadingStatus(): Promise<AdamantApiResult<GetLoadingStatusResponseDto>>;
    /**
     * Gets information on node's sync process with other peers
     */
    getSyncStatus(): Promise<AdamantApiResult<GetSyncStatusResponseDto>>;
    /**
     * Checks if the connected node is alive
     */
    getPingStatus(): Promise<AdamantApiResult<GetPingStatusResponseDto>>;
    /**
     * Gets node's software information
     */
    getNodeVersion(): Promise<AdamantApiResult<GetNodeVersionResponseDto>>;
    /**
     * Broadhash is established as an aggregated rolling hash of the past five blocks present in the database.
     */
    getBroadhash(): Promise<AdamantApiResult<GetBroadhashResponseDto>>;
    /**
     * Returns time when blockchain epoch starts. Value `2017-09-02T17:00:00.000Z` is for mainnet.
     */
    getEpoch(): Promise<AdamantApiResult<GetEpochResponseDto>>;
    /**
     * Returns current node's blockchain height
     */
    getHeight(): Promise<AdamantApiResult<GetHeightResponseDto>>;
    /**
     * Returns current fee value for `type 0` (token transfer) transactions.
     * Integer amount of 1/10^8 ADM tokens (1 ADM = 100000000).
     */
    getFee(): Promise<AdamantApiResult<GetTokenTransferFeeResponseDto>>;
    /**
     * Returns current fee values for different transaction types
     */
    getFees(): Promise<AdamantApiResult<GetTransactionTypesFeesResponseDto>>;
    /**
     * The nethash describes e.g. the Mainnet or the Testnet, that the node is connecting to.
     */
    getNethash(): Promise<AdamantApiResult<GetNethashResponseDto>>;
    /**
     * Return current slot height, which determines reward a delegate will get for forging a block.
     */
    getMilestone(): Promise<AdamantApiResult<GetMilestoneResponseDto>>;
    /**
     * Returns reward — the reward a delegate will get for forging a block.
     * Integer amount of 1/10^8 ADM tokens (1 ADM = 100000000). Depends on the slot height.
     */
    getReward(): Promise<AdamantApiResult<GetRewardResponseDto>>;
    /**
     * Returns total current supply of ADM tokens in network. Integer amount of 1/10^8 ADM tokens (1 ADM = 100000000).
     * Total supply increases with every new forged block.
     */
    getSupply(): Promise<AdamantApiResult<GetTokensTotalSupplyResponseDto>>;
    /**
     * Returns blockchain network information in a single request
     */
    getStatus(): Promise<AdamantApiResult<GetNetworkInfoResponseDto>>;
    /**
     * Returns both ADAMANT blockchain network information and Node information in a single request.
     */
    getNodeStatus(): Promise<AdamantApiResult<GetNodeStatusResponseDto>>;
    /**
     * Returns list of transactions
     */
    getTransactions(options?: TransactionQuery<TransactionsOptions>): Promise<AdamantApiResult<GetTransactionsResponseDto>>;
    /**
     * Get transaction by ID
     */
    getTransaction(id: string, options?: TransactionQuery<TransactionsOptions>): Promise<AdamantApiResult<GetTransactionByIdResponseDto>>;
    /**
     * Get `confirmed`, `uncofirmed` and `queued` transactions count
     *
     * @nav Transactions
     */
    getTransactionsCount(): Promise<AdamantApiResult<GetTransactionsCountResponseDto>>;
    /**
     * Get queued transactions count
     */
    getQueuedTransactions(): Promise<AdamantApiResult<GetQueuedTransactionsResponseDto>>;
    /**
     * Get queued transaction by ID
     */
    getQueuedTransaction(id: string): Promise<AdamantApiResult<GetQueuedTransactionsResponseDto>>;
    /**
     * Get unconfirmed transactions
     */
    getUnconfirmedTransactions(): Promise<AdamantApiResult<GetUnconfirmedTransactionsResponseDto>>;
    /**
     * Get unconfirmed transaction by ID
     */
    getUnconfirmedTransaction(id: string): Promise<AdamantApiResult<GetUnconfirmedTransactionByIdResponseDto>>;
}
export * from './generated';
export * from './utils';
