"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AdamantApi = void 0;
const axios_1 = __importStar(require("axios"));
const healthCheck_1 = require("../helpers/healthCheck");
const logger_1 = require("../helpers/logger");
const validator_1 = require("../helpers/validator");
const constants_1 = require("../helpers/constants");
const keys_1 = require("../helpers/keys");
const transactions_1 = require("../helpers/transactions");
const encryptor_1 = require("../helpers/encryptor");
const utils_1 = require("./utils");
const publicKeysCache = {};
class AdamantApi extends healthCheck_1.NodeManager {
    maxRetries;
    constructor(options) {
        const customLogger = new logger_1.Logger(options.logLevel, options.logger);
        super(customLogger, options);
        this.maxRetries = options.maxRetries ?? constants_1.DEFAULT_GET_REQUEST_RETRIES;
    }
    async request(method, endpoint, data, retryNo = 1) {
        const { logger, maxRetries } = this;
        const url = `${this.node}/api/${endpoint}`;
        try {
            const response = await (0, axios_1.default)({
                method,
                url,
                ...(method === 'POST'
                    ? {
                        data,
                    }
                    : {
                        params: data,
                    }),
            });
            return response.data;
        }
        catch (error) {
            if (error instanceof axios_1.AxiosError) {
                const { response } = error;
                const nodeStatus = response?.status
                    ? `Request to ${url} failed with ${response.status} status code`
                    : `Node ${url} hasn't returned its status`;
                const logMessage = `[ADAMANT js-api] Get-request: ${nodeStatus}, ${error}${response?.data ? '. Message: ' + response.data.toString().trim() : ''}.`;
                if (retryNo <= maxRetries) {
                    logger.log(`${logMessage} Try ${retryNo} of ${maxRetries}. Retrying…`);
                    await this.updateNodes();
                    return this.request(method, endpoint, data, retryNo + 1);
                }
                logger.warn(`${logMessage} No more attempts, returning error.`);
                return {
                    success: false,
                    errorMessage: `${error}`,
                };
            }
            return {
                success: false,
                errorMessage: `${error}`,
            };
        }
    }
    /**
     * Makes GET request to ADAMANT network.
     *
     * @details `endpoint` should be in `'accounts/getPublicKey'` format, excluding `'/api/'`
     */
    async get(endpoint, params) {
        return this.request('GET', endpoint, params);
    }
    /**
     * Makes POST request to ADAMANT network.
     *
     * @details `endpoint` should be in `'accounts/getPublicKey'` format, excluding `'/api/'`
     */
    async post(endpoint, options) {
        return this.request('POST', endpoint, options);
    }
    /**
     * Get account Public Key
     */
    async getPublicKey(address) {
        const cached = publicKeysCache[address];
        if (cached) {
            return cached;
        }
        const response = await this.get('accounts/getPublicKey', {
            address,
        });
        if (response.success) {
            const { publicKey } = response;
            publicKeysCache[address] = publicKey;
            return publicKey;
        }
        this.logger.warn(`[ADAMANT js-api] Failed to get public key for ${address}. ${response.errorMessage}.`);
        return '';
    }
    /**
     * Register new delegate within given username
     *
     * @param username The new delegate's username
     */
    async newDelegate(passphrase, username) {
        if (!(0, validator_1.isPassphrase)(passphrase)) {
            return (0, validator_1.badParameter)('passphrase');
        }
        if (!(0, validator_1.isDelegateName)(username)) {
            return (0, validator_1.badParameter)('username');
        }
        const keyPair = (0, keys_1.createKeypairFromPassphrase)(passphrase);
        const data = {
            keyPair,
            username,
        };
        const transaction = (0, transactions_1.createDelegateTransaction)(data);
        return this.post('delegates', transaction);
    }
    /**
     * Encrypt a message, creates Message transaction, signs it, and broadcasts to ADAMANT network.
     */
    async sendMessage(passphrase, addressOrPublicKey, message, type = constants_1.MessageType.Chat, amount, isAmountInADM) {
        if (!(0, validator_1.isPassphrase)(passphrase)) {
            return (0, validator_1.badParameter)('passphrase');
        }
        let address;
        let publicKey = '';
        if (!(0, validator_1.isAdmAddress)(addressOrPublicKey)) {
            if (!(0, validator_1.isAdmPublicKey)(addressOrPublicKey)) {
                return (0, validator_1.badParameter)('addressOrPublicKey', addressOrPublicKey);
            }
            publicKey = addressOrPublicKey;
            try {
                address = (0, keys_1.createAddressFromPublicKey)(publicKey);
            }
            catch (error) {
                return (0, validator_1.badParameter)('addressOrPublicKey', addressOrPublicKey);
            }
        }
        else {
            address = addressOrPublicKey;
        }
        if (!(0, validator_1.isMessageType)(type)) {
            return (0, validator_1.badParameter)('type', type);
        }
        const result = (0, validator_1.validateMessage)(message, type);
        if (!result.success) {
            return (0, validator_1.badParameter)('message', message, result.error);
        }
        let amountInSat;
        if (amount) {
            amountInSat = amount;
            if (isAmountInADM) {
                amountInSat = (0, validator_1.admToSats)(amount);
            }
            if (!(0, validator_1.isIntegerAmount)(amountInSat)) {
                return (0, validator_1.badParameter)('amount', amount);
            }
        }
        if (!publicKey) {
            publicKey = await this.getPublicKey(address);
        }
        if (!publicKey) {
            return {
                success: false,
                errorMessage: `Unable to get public key for ${addressOrPublicKey}. It is necessary for sending an encrypted message. Account may be uninitialized (https://medium.com/adamant-im/chats-and-uninitialized-accounts-in-adamant-5035438e2fcd), or network error`,
            };
        }
        const keyPair = (0, keys_1.createKeypairFromPassphrase)(passphrase);
        const encryptedMessage = (0, encryptor_1.encodeMessage)(message, keyPair, publicKey);
        const data = {
            ...encryptedMessage,
            keyPair,
            recipientId: address,
            message_type: type,
            amount: amountInSat,
        };
        const transaction = (0, transactions_1.createChatTransaction)(data);
        return this.post('transactions/process', {
            transaction,
        });
    }
    /**
     * Send tokens to an account
     */
    sendTokens(passphrase, addressOrPublicKey, amount, isAmountInADM = true) {
        if (!(0, validator_1.isPassphrase)(passphrase)) {
            return (0, validator_1.badParameter)('passphrase');
        }
        let address;
        if ((0, validator_1.isAdmAddress)(addressOrPublicKey)) {
            address = addressOrPublicKey;
        }
        else {
            if (!(0, validator_1.isAdmPublicKey)(addressOrPublicKey)) {
                return (0, validator_1.badParameter)('addressOrPublicKey', addressOrPublicKey);
            }
            try {
                address = (0, keys_1.createAddressFromPublicKey)(addressOrPublicKey);
            }
            catch (error) {
                return (0, validator_1.badParameter)('addressOrPublicKey', addressOrPublicKey);
            }
        }
        let amountInSat = amount;
        if (isAmountInADM) {
            amountInSat = (0, validator_1.admToSats)(amount);
        }
        if (!(0, validator_1.isIntegerAmount)(amountInSat)) {
            return (0, validator_1.badParameter)('amount', amount);
        }
        const keyPair = (0, keys_1.createKeypairFromPassphrase)(passphrase);
        const data = {
            keyPair,
            recipientId: address,
            amount: amountInSat,
        };
        const transaction = (0, transactions_1.createSendTransaction)(data);
        return this.post('transactions/process', {
            transaction,
        });
    }
    /**
     * Vote for specific delegates
     *
     * @param votes Array with public keys. For upvote, add leading `+` to delegate's public key. For downvote, add leading `-` to delegate's public key.
     *
     * @example
     * ```
     * voteForDelegate([
     *   '+b3d0c0b99f64d0960324089eb678e90d8bcbb3dd8c73ee748e026f8b9a5b5468',
     *   '-9ef1f6212ae871716cfa2d04e3dc5339e8fe75f89818be21ee1d75004983e2a8'
     * ])
     * ```
     */
    async voteForDelegate(passphrase, votes) {
        if (!(0, validator_1.isPassphrase)(passphrase)) {
            return (0, validator_1.badParameter)('passphrase');
        }
        const keyPair = (0, keys_1.createKeypairFromPassphrase)(passphrase);
        const uniqueVotes = {};
        for (const vote of votes) {
            const [name, direction] = (0, utils_1.parseVote)(vote);
            const cachedPublicKey = publicKeysCache[name];
            if (cachedPublicKey) {
                uniqueVotes[cachedPublicKey] = direction;
                continue;
            }
            if ((0, validator_1.isAdmVoteForAddress)(vote)) {
                const response = await this.getAccountInfo({ address: name });
                if (!response.success) {
                    this.logger.warn(`[ADAMANT js-api] Failed to get public key for ${vote}. ${response.errorMessage}.`);
                    return (0, validator_1.badParameter)('votes');
                }
                const { publicKey } = response.account;
                publicKeysCache[name] = publicKey;
                uniqueVotes[publicKey] = direction;
                continue;
            }
            if ((0, validator_1.isAdmVoteForDelegateName)(name)) {
                const response = await this.getDelegate({ username: name });
                if (!response.success) {
                    this.logger.warn(`[ADAMANT js-api] Failed to get public key for ${vote}. ${response.errorMessage}.`);
                    return (0, validator_1.badParameter)('votes');
                }
                const { publicKey } = response.delegate;
                publicKeysCache[name] = publicKey;
                uniqueVotes[publicKey] = direction;
                continue;
            }
            if (!(0, validator_1.isAdmVoteForPublicKey)(name)) {
                return (0, validator_1.badParameter)('votes');
            }
            uniqueVotes[name] = direction;
        }
        const data = {
            keyPair,
            votes: Object.keys(uniqueVotes).map(name => `${uniqueVotes[name]}${name}`),
        };
        const transaction = (0, transactions_1.createVoteTransaction)(data);
        return this.post('accounts/delegates', transaction);
    }
    /**
     * Get account information by ADAMANT address or Public Key
     */
    async getAccountInfo(options) {
        return this.get('accounts', options);
    }
    /**
     * Get account balance
     */
    async getAccountBalance(address) {
        return this.get('accounts/getBalance', {
            address,
        });
    }
    /**
     * Get block information by ID
     */
    async getBlock(id) {
        return this.get('blocks/get', { id });
    }
    /**
     * Get list of blocks
     */
    async getBlocks(options) {
        return this.get('blocks', options);
    }
    /**
     * Get list of Chats
     */
    async getChats(address, options) {
        return this.get(`chatrooms/${address}`, (0, utils_1.transformTransactionQuery)(options));
    }
    /**
     * Get messages between two accounts
     */
    async getChatMessages(address1, address2, query) {
        return this.get(`chatrooms/${address1}/${address2}`, (0, utils_1.transformTransactionQuery)(query));
    }
    /**
     * Retrieves list of registered ADAMANT delegates
     */
    async getDelegates(options) {
        return this.get('delegates', options);
    }
    /**
     * Get delegate info by `username` or `publicKey`
     */
    async getDelegate(options) {
        return this.get('delegates/get', options);
    }
    /**
     * Search delegates by `username`
     *
     * @param q - username
     */
    async searchDelegates(q) {
        return this.get('delegates/search', { q });
    }
    /**
     * Get total count of delegates
     */
    async getDelegatesCount() {
        return this.get('delegates/count');
    }
    /**
     * Get forging activity of a delegate
     */
    async getDelegateStats(generatorPublicKey) {
        return this.get('delegates/forging/getForgedByAccount', { generatorPublicKey });
    }
    /**
     * Returns list of next forgers
     *
     * @param limit count to retrieve
     */
    async getNextForgers(limit) {
        return this.get('delegates/getNextForgers', {
            limit,
        });
    }
    /**
     * Gets list of delegate's voters
     *
     * @param publicKey representing delegate's publicKey
     */
    async getVoters(publicKey) {
        return this.get('delegates/voters', { publicKey });
    }
    /**
     * Gets current votes of specific ADAMANT account
     *
     * @param address address of the account to get votes
     */
    async getVoteData(address) {
        return this.get('accounts/delegates', {
            address,
        });
    }
    /**
     * Gets list of connected peer nodes
     */
    async getPeers() {
        return this.get('peers');
    }
    /**
     * Gets loading status
     */
    async getLoadingStatus() {
        return this.get('loader/status');
    }
    /**
     * Gets information on node's sync process with other peers
     */
    async getSyncStatus() {
        return this.get('loader/status/sync');
    }
    /**
     * Checks if the connected node is alive
     */
    async getPingStatus() {
        return this.get('loader/status/ping');
    }
    /**
     * Gets node's software information
     */
    async getNodeVersion() {
        return this.get('peers/version');
    }
    /**
     * Broadhash is established as an aggregated rolling hash of the past five blocks present in the database.
     */
    async getBroadhash() {
        return this.get('blocks/getBroadhash');
    }
    /**
     * Returns time when blockchain epoch starts. Value `2017-09-02T17:00:00.000Z` is for mainnet.
     */
    async getEpoch() {
        return this.get('blocks/getEpoch');
    }
    /**
     * Returns current node's blockchain height
     */
    async getHeight() {
        return this.get('blocks/getHeight');
    }
    /**
     * Returns current fee value for `type 0` (token transfer) transactions.
     * Integer amount of 1/10^8 ADM tokens (1 ADM = 100000000).
     */
    async getFee() {
        return this.get('blocks/getFee');
    }
    /**
     * Returns current fee values for different transaction types
     */
    async getFees() {
        return this.get('blocks/getFees');
    }
    /**
     * The nethash describes e.g. the Mainnet or the Testnet, that the node is connecting to.
     */
    async getNethash() {
        return this.get('blocks/getNethash');
    }
    /**
     * Return current slot height, which determines reward a delegate will get for forging a block.
     */
    async getMilestone() {
        return this.get('blocks/getMilestone');
    }
    /**
     * Returns reward — the reward a delegate will get for forging a block.
     * Integer amount of 1/10^8 ADM tokens (1 ADM = 100000000). Depends on the slot height.
     */
    async getReward() {
        return this.get('blocks/getReward');
    }
    /**
     * Returns total current supply of ADM tokens in network. Integer amount of 1/10^8 ADM tokens (1 ADM = 100000000).
     * Total supply increases with every new forged block.
     */
    async getSupply() {
        return this.get('blocks/getSupply');
    }
    /**
     * Returns blockchain network information in a single request
     */
    async getStatus() {
        return this.get('blocks/getStatus');
    }
    /**
     * Returns both ADAMANT blockchain network information and Node information in a single request.
     */
    async getNodeStatus() {
        return this.get('node/status');
    }
    /**
     * Returns list of transactions
     */
    async getTransactions(options) {
        return this.get('transactions', (0, utils_1.transformTransactionQuery)(options));
    }
    /**
     * Get transaction by ID
     */
    async getTransaction(id, options) {
        return this.get('transactions/get', {
            id,
            ...(0, utils_1.transformTransactionQuery)(options),
        });
    }
    /**
     * Get `confirmed`, `uncofirmed` and `queued` transactions count
     *
     * @nav Transactions
     */
    async getTransactionsCount() {
        return this.get('transactions/count');
    }
    /**
     * Get queued transactions count
     */
    async getQueuedTransactions() {
        return this.get('transactions/queued');
    }
    /**
     * Get queued transaction by ID
     */
    async getQueuedTransaction(id) {
        return this.get('transactions/queued/get', { id });
    }
    /**
     * Get unconfirmed transactions
     */
    async getUnconfirmedTransactions() {
        return this.get('transactions/unconfirmed');
    }
    /**
     * Get unconfirmed transaction by ID
     */
    async getUnconfirmedTransaction(id) {
        return this.get('transactions/unconfirmed/get', { id });
    }
}
exports.AdamantApi = AdamantApi;
__exportStar(require("./generated"), exports);
__exportStar(require("./utils"), exports);
