"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.decodeMessage = exports.encodeMessage = exports.utf8ArrayToStr = exports.hexToBytes = exports.bytesToHex = void 0;
const sodium_browserify_tweetnacl_1 = __importDefault(require("sodium-browserify-tweetnacl"));
const tweetnacl_1 = __importDefault(require("tweetnacl"));
const ed2curve_1 = __importDefault(require("ed2curve"));
const keys_1 = require("./keys");
const bytesToHex = (bytes) => {
    let hex = '';
    for (const byte of bytes) {
        hex += (byte >>> 4).toString(16);
        hex += (byte & 0xf).toString(16);
    }
    return hex;
};
exports.bytesToHex = bytesToHex;
const hexToBytes = (hex) => {
    const bytes = [];
    for (let c = 0; c < hex.length; c += 2) {
        bytes.push(parseInt(hex.slice(c, c + 2), 16));
    }
    return Uint8Array.from(bytes);
};
exports.hexToBytes = hexToBytes;
const utf8ArrayToStr = (array) => {
    const len = array.length;
    let out = '';
    let i = 0;
    let c;
    let char2;
    let char3;
    while (i < len) {
        c = array[i++];
        switch (c >> 4) {
            case 0:
            case 1:
            case 2:
            case 3:
            case 4:
            case 5:
            case 6:
            case 7:
                // 0xxxxxxx
                out += String.fromCharCode(c);
                break;
            case 12:
            case 13:
                // 110x xxxx   10xx xxxx
                char2 = array[i++];
                out += String.fromCharCode(((c & 0x1f) << 6) | (char2 & 0x3f));
                break;
            case 14:
                // 1110 xxxx  10xx xxxx  10xx xxxx
                char2 = array[i++];
                char3 = array[i++];
                out += String.fromCharCode(((c & 0x0f) << 12) | ((char2 & 0x3f) << 6) | ((char3 & 0x3f) << 0));
                break;
        }
    }
    return out;
};
exports.utf8ArrayToStr = utf8ArrayToStr;
const encodeMessage = (msg, keypair, recipientPublicKey) => {
    const nonce = Buffer.allocUnsafe(24);
    sodium_browserify_tweetnacl_1.default.randombytes(nonce);
    const plainText = Buffer.from(msg.toString());
    const DHSecretKey = ed2curve_1.default.convertSecretKey(keypair.privateKey);
    let publicKey = recipientPublicKey;
    if (typeof publicKey === 'string') {
        publicKey = (0, exports.hexToBytes)(publicKey);
    }
    const DHPublicKey = ed2curve_1.default.convertPublicKey(publicKey);
    if (!DHPublicKey) {
        throw new Error('encodeMessage: invalid key');
    }
    const encrypted = tweetnacl_1.default.box(plainText, nonce, DHPublicKey, DHSecretKey);
    return {
        message: (0, exports.bytesToHex)(encrypted),
        own_message: (0, exports.bytesToHex)(nonce),
    };
};
exports.encodeMessage = encodeMessage;
const decodeMessage = (message, senderPublicKey, keyPairOrPassphrase, nonce) => {
    const keypair = typeof keyPairOrPassphrase === 'string'
        ? (0, keys_1.createKeypairFromPassphrase)(keyPairOrPassphrase)
        : keyPairOrPassphrase;
    const publicKey = typeof senderPublicKey === 'string'
        ? (0, exports.hexToBytes)(senderPublicKey)
        : senderPublicKey;
    if (typeof message !== 'string') {
        throw new Error('decodeMessage message should be a string');
    }
    if (typeof nonce !== 'string') {
        throw new Error('decodeMessage: nonce should be a string');
    }
    if (!(publicKey instanceof Uint8Array)) {
        throw new Error('decodeMessage: senderPublicKey should be a string or an instance of Uint8Array');
    }
    const DHPublicKey = ed2curve_1.default.convertPublicKey(publicKey);
    if (!DHPublicKey) {
        throw new Error('decodeMessage: invalid key');
    }
    const { privateKey } = keypair;
    const DHSecretKey = ed2curve_1.default.convertSecretKey(privateKey);
    const decrypted = tweetnacl_1.default.box.open((0, exports.hexToBytes)(message), (0, exports.hexToBytes)(nonce), DHPublicKey, DHSecretKey);
    return decrypted ? (0, exports.utf8ArrayToStr)(decrypted) : '';
};
exports.decodeMessage = decodeMessage;
