"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sign = exports.chatGetBytes = exports.statesGetBytes = exports.delegatesGetBytes = exports.voteGetBytes = exports.signTransaction = exports.getBytes = exports.getAssetBytes = exports.getHash = void 0;
const sodium_browserify_tweetnacl_1 = __importDefault(require("sodium-browserify-tweetnacl"));
const bytebuffer_1 = __importDefault(require("bytebuffer"));
const crypto_1 = __importDefault(require("crypto"));
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const bignumber_1 = require("../bignumber");
const constants_1 = require("../constants");
function getHash(trs, options = { skipSignature: true }) {
    const hash = crypto_1.default
        .createHash('sha256')
        .update(getBytes(trs, options.skipSignature))
        .digest();
    return hash;
}
exports.getHash = getHash;
function getAssetBytes(transaction) {
    const { type } = transaction;
    const { VOTE, DELEGATE, CHAT_MESSAGE, STATE } = constants_1.TransactionType;
    let assetBytes = null;
    if (type === VOTE) {
        assetBytes = voteGetBytes(transaction);
    }
    else if (type === DELEGATE) {
        assetBytes = delegatesGetBytes(transaction);
    }
    else if (type === CHAT_MESSAGE) {
        assetBytes = chatGetBytes(transaction);
    }
    else if (type === STATE) {
        assetBytes = statesGetBytes(transaction);
    }
    return { assetBytes, assetSize: assetBytes?.length || 0 };
}
exports.getAssetBytes = getAssetBytes;
function getBytes(transaction, skipSignature = true) {
    const result = getAssetBytes(transaction);
    if (!result) {
        throw new Error('Not supported transaction type');
    }
    const { assetSize, assetBytes } = result;
    const bb = new bytebuffer_1.default(1 + 4 + 32 + 8 + 8 + 64 + 64 + assetSize, true);
    bb.writeByte(transaction.type);
    bb.writeInt(transaction.timestamp);
    const senderPublicKeyBuffer = Buffer.from(transaction.senderPublicKey, 'hex');
    for (const buf of senderPublicKeyBuffer) {
        bb.writeByte(buf);
    }
    if ('recipientId' in transaction && transaction.recipientId) {
        const bignum = new bignumber_js_1.default(transaction.recipientId.slice(1));
        const recipient = (0, bignumber_1.toBuffer)(bignum, { size: 8 });
        for (let i = 0; i < 8; i++) {
            bb.writeByte(recipient[i] || 0);
        }
    }
    else {
        for (let i = 0; i < 8; i++) {
            bb.writeByte(0);
        }
    }
    bb.writeLong(transaction.amount);
    if (assetBytes && assetSize > 0) {
        for (const assetByte of assetBytes) {
            bb.writeByte(assetByte);
        }
    }
    if (!skipSignature && transaction.signature) {
        const signatureBuffer = Buffer.from(transaction.signature, 'hex');
        for (let i = 0; i < signatureBuffer.length; i++) {
            bb.writeByte(signatureBuffer[i]);
        }
    }
    bb.flip();
    const arrayBuffer = new Uint8Array(bb.toArrayBuffer());
    const buffer = [];
    for (let i = 0; i < arrayBuffer.length; i++) {
        buffer[i] = arrayBuffer[i];
    }
    return Buffer.from(buffer);
}
exports.getBytes = getBytes;
function signTransaction(trs, keypair) {
    const hash = getHash(trs);
    return sign(hash, keypair).toString('hex');
}
exports.signTransaction = signTransaction;
function voteGetBytes(trs) {
    const { votes } = trs.asset;
    return votes ? Buffer.from(votes.join(''), 'utf8') : null;
}
exports.voteGetBytes = voteGetBytes;
function delegatesGetBytes(trs) {
    const { username } = trs.asset.delegate;
    return username ? Buffer.from(username, 'utf8') : null;
}
exports.delegatesGetBytes = delegatesGetBytes;
function statesGetBytes(trs) {
    const { value } = trs.asset.state;
    if (!value) {
        return null;
    }
    let buf = Buffer.from([]);
    const { key, type } = trs.asset.state;
    const stateBuf = Buffer.from(value);
    buf = Buffer.concat([buf, stateBuf]);
    if (key) {
        const keyBuf = Buffer.from(key);
        buf = Buffer.concat([buf, keyBuf]);
    }
    const bb = new bytebuffer_1.default(4 + 4, true);
    bb.writeInt(type);
    bb.flip();
    buf = Buffer.concat([buf, bb.toBuffer()]);
    return buf;
}
exports.statesGetBytes = statesGetBytes;
function chatGetBytes(trs) {
    let buf = Buffer.from([]);
    const { message } = trs.asset.chat;
    const messageBuf = Buffer.from(message, 'hex');
    buf = Buffer.concat([buf, messageBuf]);
    const { own_message: ownMessage } = trs.asset.chat;
    if (ownMessage) {
        const ownMessageBuf = Buffer.from(ownMessage, 'hex');
        buf = Buffer.concat([buf, ownMessageBuf]);
    }
    const bb = new bytebuffer_1.default(4 + 4, true);
    bb.writeInt(trs.asset.chat.type);
    bb.flip();
    buf = Buffer.concat([buf, Buffer.from(bb.toBuffer())]);
    return buf;
}
exports.chatGetBytes = chatGetBytes;
function sign(hash, keypair) {
    const sign = sodium_browserify_tweetnacl_1.default.crypto_sign_detached(hash, Buffer.from(keypair.privateKey));
    return sign;
}
exports.sign = sign;
