"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.badParameter = exports.admToSats = exports.isDelegateName = exports.validateMessage = exports.isMessageType = exports.isStringAmount = exports.isIntegerAmount = exports.isAdmVoteForDelegateName = exports.isAdmVoteForAddress = exports.isAdmVoteForPublicKey = exports.isAdmPublicKey = exports.isAdmAddress = exports.isEndpoint = exports.isPassphrase = exports.parseJsonSafe = exports.isNumeric = exports.getRandomIntInclusive = void 0;
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const constants_1 = require("./constants");
const getRandomIntInclusive = (minimum, maximum) => {
    const min = Math.ceil(minimum);
    const max = Math.floor(maximum);
    // The maximum is inclusive and the minimum is inclusive
    return Math.floor(Math.random() * (max - min + 1) + min);
};
exports.getRandomIntInclusive = getRandomIntInclusive;
const isNumeric = (str) => typeof str === 'string' && !isNaN(parseFloat(str));
exports.isNumeric = isNumeric;
const parseJsonSafe = (json) => {
    try {
        const result = JSON.parse(json);
        return {
            result,
            success: true,
        };
    }
    catch (error) {
        return {
            error,
            success: false,
        };
    }
};
exports.parseJsonSafe = parseJsonSafe;
const isPassphrase = (passphrase) => typeof passphrase === 'string' && passphrase.length > 30;
exports.isPassphrase = isPassphrase;
const isEndpoint = (endpoint) => typeof endpoint === 'string' && endpoint.startsWith('/');
exports.isEndpoint = isEndpoint;
const RE_ADM_ADDRESS = /^U([0-9]{6,})$/;
const isAdmAddress = (address) => typeof address === 'string' && RE_ADM_ADDRESS.test(address);
exports.isAdmAddress = isAdmAddress;
const RE_HEX = /^[a-fA-F0-9]+$/;
const isAdmPublicKey = (publicKey) => typeof publicKey === 'string' &&
    publicKey.length === 64 &&
    RE_HEX.test(publicKey);
exports.isAdmPublicKey = isAdmPublicKey;
const RE_ADM_VOTE_FOR_PUBLIC_KEY = /^(\+|-)[a-fA-F0-9]{64}$/;
const isAdmVoteForPublicKey = (publicKey) => typeof publicKey === 'string' && RE_ADM_VOTE_FOR_PUBLIC_KEY.test(publicKey);
exports.isAdmVoteForPublicKey = isAdmVoteForPublicKey;
const RE_ADM_VOTE_FOR_ADDRESS = /^(\+|-)U([0-9]{6,})$/;
const isAdmVoteForAddress = (address) => typeof address === 'string' && RE_ADM_VOTE_FOR_ADDRESS.test(address);
exports.isAdmVoteForAddress = isAdmVoteForAddress;
const RE_ADM_VOTE_FOR_DELEGATE_NAME = /^(\+|-)([a-z0-9!@$&_]{1,20})$/;
const isAdmVoteForDelegateName = (delegateName) => typeof delegateName === 'string' &&
    RE_ADM_VOTE_FOR_DELEGATE_NAME.test(delegateName);
exports.isAdmVoteForDelegateName = isAdmVoteForDelegateName;
const isIntegerAmount = (amount) => Number.isSafeInteger(amount);
exports.isIntegerAmount = isIntegerAmount;
const isStringAmount = (amount) => (0, exports.isNumeric)(amount);
exports.isStringAmount = isStringAmount;
const isMessageType = (messageType) => [1, 2, 3].includes(messageType);
exports.isMessageType = isMessageType;
const validateMessage = (message, messageType = constants_1.MessageType.Chat) => {
    if (typeof message !== 'string') {
        return {
            success: false,
            error: 'Message should be a string',
        };
    }
    if ([constants_1.MessageType.Rich, constants_1.MessageType.Signal].includes(messageType)) {
        const data = (0, exports.parseJsonSafe)(message);
        const { success, result } = data;
        if (!success || typeof result !== 'object') {
            return {
                success: false,
                error: "For rich and signal message, 'message' should be a JSON string",
            };
        }
        const typeInLowerCase = result.type?.toLowerCase();
        if (typeInLowerCase?.includes('_transaction') &&
            typeInLowerCase !== result.type) {
            return {
                success: false,
                error: "Value '<coin>_transaction' must be in lower case",
            };
        }
        if (typeof result.amount !== 'string' || !(0, exports.isStringAmount)(result.amount)) {
            return {
                success: false,
                error: "Field 'amount' must be a string, representing a number",
            };
        }
    }
    return { success: true };
};
exports.validateMessage = validateMessage;
const RE_ADM_DELEGATE_NAME = /^[a-z0-9!@$&_]{1,20}$/;
const isDelegateName = (name) => typeof name === 'string' && RE_ADM_DELEGATE_NAME.test(name);
exports.isDelegateName = isDelegateName;
const admToSats = (amount) => new bignumber_js_1.default(String(amount)).multipliedBy(constants_1.SAT).integerValue().toNumber();
exports.admToSats = admToSats;
const badParameter = (name, value, message) => ({
    success: false,
    error: `Wrong '${name}' parameter${value ? `: ${value}` : ''}${message ? `. Error: ${message}` + message : ''}`,
});
exports.badParameter = badParameter;
