import type { ActiveNode } from './healthCheck';
import { Logger } from './logger';
import { TransactionType } from './constants';
import type { AnyTransaction, ChatMessageTransaction, KVSTransaction, RegisterDelegateTransaction, TokenTransferTransaction, VoteForDelegateTransaction } from '../api/generated';
import type { AdamantAddress } from '../api';
export type WsType = 'ws' | 'wss';
export interface WsOptions {
    /**
     * ADM address to subscribe to notifications
     */
    admAddress: AdamantAddress;
    /**
     * Websocket type: `'wss'` or `'ws'`. `'wss'` is recommended.
     */
    wsType?: WsType;
    /**
     * Must connect to node with minimum ping. Not recommended. Default is `false`.
     */
    useFastest?: boolean;
    logger?: Logger;
}
type ErrorHandler = (error: unknown) => void;
type TransactionMap = {
    [TransactionType.SEND]: TokenTransferTransaction;
    [TransactionType.DELEGATE]: RegisterDelegateTransaction;
    [TransactionType.VOTE]: VoteForDelegateTransaction;
    [TransactionType.CHAT_MESSAGE]: ChatMessageTransaction;
    [TransactionType.STATE]: KVSTransaction;
};
type EventType = keyof TransactionMap;
export type TransactionHandler<T extends AnyTransaction> = (transaction: T) => void;
export type SingleTransactionHandler = TransactionHandler<TokenTransferTransaction> | TransactionHandler<RegisterDelegateTransaction> | TransactionHandler<VoteForDelegateTransaction> | TransactionHandler<ChatMessageTransaction> | TransactionHandler<KVSTransaction>;
export type AnyTransactionHandler = TransactionHandler<AnyTransaction>;
export declare class WebSocketClient {
    /**
     * Web socket client options.
     */
    options: WsOptions;
    /**
     * Current socket connection
     */
    private connection?;
    /**
     * List of nodes that are active, synced and support socket.
     */
    private nodes;
    private logger;
    private errorHandler;
    private eventHandlers;
    constructor(options: WsOptions);
    /**
     * Filters nodes that support websocket and connects to one of them.
     *
     * @param nodes Sorted by ping array of active nodes
     */
    reviseConnection(nodes: ActiveNode[]): void;
    /**
     * Chooses node and sets up connection.
     */
    setConnection(): void;
    /**
     * Sets an error handler for all event handlers.
     *
     * @example
     * ```js
     * socket.onMessage(() => throw new Error('catch me'))
     *
     * socket.catch((error) => {
     *   console.log(error) // Error: catch me
     * })
     * ```
     */
    catch(callback: ErrorHandler): this;
    /**
     * Removes the handler from all types.
     */
    off(handler: SingleTransactionHandler): this;
    /**
     * Adds an event listener handler for all transaction types.
     */
    on(handler: AnyTransactionHandler): this;
    /**
     * Adds an event listener handler for the specific transaction types.
     */
    on<T extends EventType>(types: T | T[], handler: TransactionHandler<TransactionMap[T]>): this;
    /**
     * Registers an event handler for Chatn Message transactions.
     */
    onMessage(handler: TransactionHandler<ChatMessageTransaction>): this;
    /**
     * Registers an event handler for Token Transfer transactions.
     */
    onTransfer(handler: TransactionHandler<TokenTransferTransaction>): this;
    /**
     * Registers an event handler for Register Delegate transactions.
     */
    onNewDelegate(handler: TransactionHandler<RegisterDelegateTransaction>): this;
    /**
     * Registers an event handler for Vote for Delegate transactions.
     */
    onVoteForDelegate(handler: TransactionHandler<VoteForDelegateTransaction>): this;
    /**
     * Registers an event handler for Key-Value Store (KVS) transactions.
     */
    onKVS(handler: TransactionHandler<KVSTransaction>): this;
    private handle;
    /**
     * Chooses fastest or random node based on {@link WsOptions.useFastest} option
     *
     * @returns WebSocket url
     */
    chooseNode(): string;
    fastestNode(): ActiveNode;
    randomNode(): ActiveNode;
}
export {};
