"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebSocketClient = void 0;
const socket_io_client_1 = require("socket.io-client");
const logger_1 = require("./logger");
const validator_1 = require("./validator");
const constants_1 = require("./constants");
class WebSocketClient {
    /**
     * Web socket client options.
     */
    options;
    /**
     * Current socket connection
     */
    connection;
    /**
     * List of nodes that are active, synced and support socket.
     */
    nodes;
    logger;
    errorHandler;
    eventHandlers = {
        [constants_1.TransactionType.SEND]: [],
        [constants_1.TransactionType.DELEGATE]: [],
        [constants_1.TransactionType.VOTE]: [],
        [constants_1.TransactionType.CHAT_MESSAGE]: [],
        [constants_1.TransactionType.STATE]: [],
    };
    constructor(options) {
        this.logger = options.logger || new logger_1.Logger();
        this.options = {
            wsType: 'ws',
            ...options,
        };
        this.nodes = [];
        this.errorHandler = (error) => {
            this.logger.error(`${error}`);
        };
    }
    /**
     * Filters nodes that support websocket and connects to one of them.
     *
     * @param nodes Sorted by ping array of active nodes
     */
    reviseConnection(nodes) {
        if (this.connection?.connected) {
            return;
        }
        const { wsType } = this.options;
        this.nodes = nodes.filter(node => node.socketSupport &&
            !node.outOfSync &&
            // Remove nodes without IP if 'ws' connection type
            (wsType !== 'ws' || !node.isHttps || node.ip));
        this.setConnection();
    }
    /**
     * Chooses node and sets up connection.
     */
    setConnection() {
        const { logger } = this;
        const supportedCount = this.nodes.length;
        if (!supportedCount) {
            logger.warn('[Socket] No supported socket nodes at the moment.');
            return;
        }
        const node = this.chooseNode();
        logger.log(`[Socket] Supported nodes: ${supportedCount}. Connecting to ${node}...`);
        const connection = (0, socket_io_client_1.io)(node, {
            reconnection: false,
            timeout: 5000,
        });
        connection.on('connect', () => {
            const { admAddress } = this.options;
            connection.emit('address', admAddress);
            logger.info(`[Socket] Connected to ${node} and subscribed to incoming transactions for ${admAddress}`);
        });
        connection.on('disconnect', reason => logger.warn(`[Socket] Disconnected. Reason: ${reason}`));
        connection.on('connect_error', error => logger.warn(`[Socket] Connection error: ${error}`));
        connection.on('newTrans', (transaction) => {
            if (transaction.recipientId !== this.options.admAddress) {
                return;
            }
            this.handle(transaction);
        });
        this.connection = connection;
    }
    /**
     * Sets an error handler for all event handlers.
     *
     * @example
     * ```js
     * socket.onMessage(() => throw new Error('catch me'))
     *
     * socket.catch((error) => {
     *   console.log(error) // Error: catch me
     * })
     * ```
     */
    catch(callback) {
        this.errorHandler = callback;
        return this;
    }
    /**
     * Removes the handler from all types.
     */
    off(handler) {
        for (const handlers of Object.values(this.eventHandlers)) {
            const index = handlers.indexOf(handler);
            if (index !== -1) {
                handlers.splice(index, 1);
            }
        }
        return this;
    }
    on(typesOrHandler, handler) {
        if (handler === undefined) {
            if (typeof typesOrHandler === 'function') {
                for (const trigger of Object.keys(this.eventHandlers)) {
                    this.eventHandlers[+trigger].push(typesOrHandler);
                }
            }
        }
        else {
            const triggers = Array.isArray(typesOrHandler)
                ? typesOrHandler
                : [typesOrHandler];
            for (const trigger of triggers) {
                this.eventHandlers[trigger].push(handler);
            }
        }
        return this;
    }
    /**
     * Registers an event handler for Chatn Message transactions.
     */
    onMessage(handler) {
        return this.on(constants_1.TransactionType.CHAT_MESSAGE, handler);
    }
    /**
     * Registers an event handler for Token Transfer transactions.
     */
    onTransfer(handler) {
        return this.on(constants_1.TransactionType.SEND, handler);
    }
    /**
     * Registers an event handler for Register Delegate transactions.
     */
    onNewDelegate(handler) {
        return this.on(constants_1.TransactionType.DELEGATE, handler);
    }
    /**
     * Registers an event handler for Vote for Delegate transactions.
     */
    onVoteForDelegate(handler) {
        return this.on(constants_1.TransactionType.VOTE, handler);
    }
    /**
     * Registers an event handler for Key-Value Store (KVS) transactions.
     */
    onKVS(handler) {
        return this.on(constants_1.TransactionType.STATE, handler);
    }
    async handle(transaction) {
        const handlers = this.eventHandlers[transaction.type];
        for (const handler of handlers) {
            try {
                await handler(transaction);
            }
            catch (error) {
                this.errorHandler(error);
            }
        }
    }
    /**
     * Chooses fastest or random node based on {@link WsOptions.useFastest} option
     *
     * @returns WebSocket url
     */
    chooseNode() {
        const { wsType, useFastest } = this.options;
        const node = useFastest ? this.fastestNode() : this.randomNode();
        let baseURL;
        if (wsType === 'ws') {
            const host = node.ip ? node.ip : node.baseURL;
            baseURL = `${host}:${node.wsPort}`;
        }
        else {
            baseURL = node.baseURL; // no port if wss
        }
        return `${wsType}://${baseURL}`;
    }
    fastestNode() {
        return this.nodes[0]; // They are already sorted by ping
    }
    randomNode() {
        const randomIndex = (0, validator_1.getRandomIntInclusive)(0, this.nodes.length - 1);
        return this.nodes[randomIndex];
    }
}
exports.WebSocketClient = WebSocketClient;
